Package: IRTM
Title: Theory-Driven Item Response Theory (IRT) Models
Version: 0.0.1.2
Authors@R: 
    c(person(given = "Marco",
           family = "Morucci",
           role = "aut",
           email = "moruccim@msu.edu"),
    person(given = "Margaret",
           family = "Foster",
           role = "aut",
            email = "m.jenkins.foster@gmail.com",
           comment = c(ORCID = "0000-0002-6418-8394")),
    person(given = "David",
           family = "Siegel",
           role = c("aut", "cre"),
	   email = "david.siegel@duke.edu",
           comment = c(ORCID = "0000-0003-1619-6119")))
Description: IRT-M is a semi-supervised approach based on Bayesian Item Response Theory
    that produces theoretically identified underlying dimensions from input data
    and a constraints matrix. The methodology is fully described in 'Morucci et al. (2024),
    "Measurement That Matches Theory: Theory-Driven Identification in Item Response Theory Models"'. 
    Details are available at <https://www.cambridge.org/core/journals/american-political-science-review/article/measurement-that-matches-theory-theorydriven-identification-in-item-response-theory-models/395DA1DFE3DCD7B866DC053D7554A30B>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: truncnorm, tmvtnorm, utils, RcppProgress, RcppDist, ggplot2, R
        (>= 3.5.0)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), RColorBrewer,
        fastDummies, ggrepel, tidyverse, spelling
VignetteBuilder: knitr
LazyData: true
LinkingTo: Rcpp, RcppArmadillo, RcppDist, RcppProgress
Imports: coda, Rcpp, RcppArmadillo, ggridges, rlang, dplyr, reshape2,
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-04-20 12:46:32 UTC; Promachos
Author: Marco Morucci [aut],
  Margaret Foster [aut] (ORCID: <https://orcid.org/0000-0002-6418-8394>),
  David Siegel [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1619-6119>)
Maintainer: David Siegel <david.siegel@duke.edu>
Repository: CRAN
Date/Publication: 2026-04-21 06:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-21 19:16:45 UTC; unix
Archs: IRTM.so.dSYM

Package: IRTM
Title: Theory-Driven Item Response Theory (IRT) Models
Version: 0.0.1.2
Authors@R: 
    c(person(given = "Marco",
           family = "Morucci",
           role = "aut",
           email = "moruccim@msu.edu"),
    person(given = "Margaret",
           family = "Foster",
           role = "aut",
            email = "m.jenkins.foster@gmail.com",
           comment = c(ORCID = "0000-0002-6418-8394")),
    person(given = "David",
           family = "Siegel",
           role = c("aut", "cre"),
	   email = "david.siegel@duke.edu",
           comment = c(ORCID = "0000-0003-1619-6119")))
Description: IRT-M is a semi-supervised approach based on Bayesian Item Response Theory
    that produces theoretically identified underlying dimensions from input data
    and a constraints matrix. The methodology is fully described in 'Morucci et al. (2024),
    "Measurement That Matches Theory: Theory-Driven Identification in Item Response Theory Models"'. 
    Details are available at <https://www.cambridge.org/core/journals/american-political-science-review/article/measurement-that-matches-theory-theorydriven-identification-in-item-response-theory-models/395DA1DFE3DCD7B866DC053D7554A30B>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: truncnorm, tmvtnorm, utils, RcppProgress, RcppDist, ggplot2, R
        (>= 3.5.0)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), RColorBrewer,
        fastDummies, ggrepel, tidyverse, spelling
VignetteBuilder: knitr
LazyData: true
LinkingTo: Rcpp, RcppArmadillo, RcppDist, RcppProgress
Imports: coda, Rcpp, RcppArmadillo, ggridges, rlang, dplyr, reshape2,
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-04-20 12:46:32 UTC; Promachos
Author: Marco Morucci [aut],
  Margaret Foster [aut] (ORCID: <https://orcid.org/0000-0002-6418-8394>),
  David Siegel [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1619-6119>)
Maintainer: David Siegel <david.siegel@duke.edu>
Repository: CRAN
Date/Publication: 2026-04-21 06:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 13:39:29 UTC; unix
Archs: IRTM.so.dSYM

# Generated by roxygen2: do not edit by hand

export(Geweke_convergence)
export(M_constrained_irt)
export(dim_corr)
export(get_lambdas)
export(irt_m)
export(irt_vis)
export(mse)
export(pair_gen_anchors)
export(standardize_theta)
export(theta_av)
export(theta_lambda_traceplots)
import(RcppProgress)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(RcppArmadillo,armadillo_version)
importFrom(RcppDist,bayeslm)
importFrom(coda,geweke.diag)
importFrom(dplyr,mutate)
importFrom(ggridges,geom_density_ridges)
importFrom(graphics,par)
importFrom(reshape2,melt)
importFrom(rlang,sym)
importFrom(stats,pnorm)
importFrom(stats,sd)
importFrom(tmvtnorm,rtmvnorm)
importFrom(truncnorm,rtruncnorm)
useDynLib(IRTM, .registration = TRUE)

# Information Retrieval & Text Mining Toolbox

This repository holds functions pivotal for IRTM processing. This repo. is staged for continuous development.

## Quick Install using 'pip/pip3' & GitHub

```bash
pip install git+https://github.com/KanishkNavale/IRTM-Toolbox.git
```

## Import Module

```python
from irtm.toolbox import *
```

## Using Functions

1. Soundex: A phonetic algorithm for indexing names by sound, as pronounced in English.

    ```python
    print(soundex('Muller'))
    print(soundex('Mueller'))
    ```

    ```bash
    >>> 'M466'
    >>> 'M466'
    ```

2. Tokenizer: Converts a sequence of characters into a sequence of tokens.

    ```python
    print(tokenize('LINUX'))
    print(tokenize('Text Mining 2021'))
    ```

    ```bash
    >>> ['linux']
    >>> ['text', 'mining']
    ```

3. Vectorize: Converts a string to token based weight tensor.

    ```python
    vector = vectorize([
            'texts ([string]): a multiline or a single line string.',
            'dict ([list], optional): list of tokens. Defaults to None.',
            'enable_Idf (bool, optional): use IDF or not. Defaults to True.',
            'normalize (str, optional): normalization of vector. Defaults to l2.',
            'max_dim ([int], optional): dimension of vector. Defaults to None.',
            'smooth (bool, optional): restricts value >0. Defaults to True.',
            'weightedTf (bool, optional): Tf = 1+log(Tf). Defaults to True.',
            'return_features (bool, optional): feature vector. Defaults to False.'
            ])

    print(f'Vector Shape={vector.shape}')
    ```

    ```bash
    >>> Vector Shape=(8, 37)
    ```

4. Predict Token Weights: Computes importance of a token based on classification optimization.

    ```python
    dictionary = ['vector', 'string', 'bool']
    vector = vectorize([
            'X ([np.array]): vectorized matrix columns arraged as per the dictionary.',
            'y ([labels]): True classification labels.',
            'epochs ([int]): Optimization epochs.',
            'verbose (bool, optional): Enable verbose outputs. Defaults to False.',
            'dict ([type], optional): list of tokens. Defaults to None.'
            ], dict=dictionary)

    labels = np.random.randint(1, size=(vector.shape[0], 1))
    weights = predict_weights(vector, labels, 100, dict=dictionary)
    ```

    ```bash
    >>> Token-Weights Mappings: {'vector': 0.22097790924850977, 
                                 'string': 0.39296369957440075, 
                                 'bool': 0.689853175081446}
    ```

5. Page Rank: Computes page rank from a chain matrix

    ```python
    chain_matrix = np.array([[0, 0, 1],
                             [1, 0, 1],
                             [0, 1, 0]])

    print(page_rank(chain_matrix))
    
    rank, TPM = page_rank(chain_matrix, return_TransMatrix=True)
    print(f'Page Rank: {rank} \nTransition Probablity Matrix: \n{TPM}')
    ```

    ```bash
    >>> [0.0047 0.997  0.0767]
    >>> Page Rank: [0.0047 0.997  0.0767] 
        Transition Probablity Matrix: 
        [[0.03333333 0.03333333 0.93333333]
        [0.48333333 0.03333333 0.48333333]
        [0.03333333 0.93333333 0.03333333]]
    ```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----packages, eval=T, echo=T, message=F, warning = FALSE---------------------
## Data prep:
library(tidyverse) # version: tidyverse_2.0.0
library(dplyr) #version: dplyr_1.1.4
library(stats) # version: stats4
library(fastDummies) # version: fastDummies_1.7.3
library(reshape2) #version: reshape2_1.4.4

## IRT-M estimation:
#devtools::install_github("dasiegel/IRT-M")
library(IRTM) #version 1.00

## Results visualization:
library(ggplot2)  # version: ggplot2_3.4.4
library(ggridges) #version: ggridges_0.5.6
library(RColorBrewer) #version: RColorBrewer_1.1-3
library(ggrepel) # version: ggrepel_0.9.5

## ----load-dat, eval=T, echo=T, message=F, warning = FALSE---------------------

synth_questions <- NULL  # Initialize to avoid R CMD check notes

load("./vdata/synth_questions.rda")
## Convert numeric ordinal responses to factors

ebdatsub <- lapply(ebdatsynth[,], factor) ## that's a list now

## converts the list back into a dataframe:
Y <- dummy_cols(.data=ebdatsub,
                       remove_selected_columns=TRUE)

## remove the .data that dummy_cols adds to the column names
colnames(Y) <- gsub(".data.", '', colnames(Y))

## remove the data objects:
rm(ebdatsub)
rm(ebdatsynth)

## ----load-mcodes, eval=T, echo=T, message=F, warning = F----------------------
load('./vdata/mcodes.rda')

## Only keep M-Codes with loadings or outcomes:
MCodes$encoding <- rowSums(abs(MCodes[,4:9]))
MCodes <- MCodes[which(MCodes$encoding > 0),]


## ----format-mcodes, eval=T, echo=T, message=F, warning = F--------------------
## Produce a K-coded questions x R-responses data frame:

d <- 6 #number of coded dimensions
mcolumns <- c("QMap", "D1-Culture threat",
              "D2-ReligionThreat",
              "D3-Economic Threat",
              "D4-HealthThreat",
              "O1-OutcomeSupportImmigration", "O2-OutcomeSupportEU")

combine <- MCodes[,mcolumns] %>% ## question codes and loadings
    inner_join(
        Y %>%
        t() %>%
        as.data.frame(stringsAsFactors = FALSE) %>%
        type_convert() %>%
        rownames_to_column(var = "question"),
        by = c("QMap" = "question"  )
    )

M_matrix <- as.data.frame(combine[, 1:(d+1)])

#Reverse the earlier transposition of the observations:
Y_in <- combine[, (d+2):ncol(combine)]%>%
    t() %>%
    as.data.frame()

Y_in <- as.data.frame(sapply(Y_in, as.numeric))

## Take the question names and
## convert to column names

question <- combine[,1] %>%
    as.data.frame()
colnames(Y_in) <- question[,1]
rm(combine)
rm(question)

## ----run-irtm, eval=T, echo=T, message=F, warning = FALSE---------------------
d=6
irt <- irt_m(Y_in = Y_in, d = d, M_matrix = M_matrix, nsamp = 1000, nburn=20, thin=1)


## ----theta-average, eval=T, echo=T, message=F, warning = FALSE----------------
avgthetas <- theta_av(irt$theta)


## ----read-idvs, eval=T, echo=T------------------------------------------------

## load idvs:
load("./vdata/synth_idvs.rda")

thetas <- cbind(avgthetas, synthidvs)

## Rename columns for readability:
colnames(thetas)[1:6] <- paste0("Theta", 1:6)
colnames(thetas)[colnames(thetas)=="qb7_2"] <- "MoreBorderControl"

## Cast into factors:
thetas$mediatrust <- as.factor(thetas$mediatrust)
thetas$class <- as.factor(thetas$class)
thetas$polorient <- as.factor(thetas$polorient)

head(thetas)


## ----corr-matrixes, eval=T, echo=T, message=F, warning = FALSE----------------

#Compute correlation matrix of latent dimensions
theta_names <- c("Culture Threat", "Religion Threat", "Economic Threat", "Health Threat", "Support Immigration", "Support EU")
theta_corr <- dim_corr(irt$Sigma, theta_names)
theta_corr


## ----viz-thetas, eval=T, echo=T, echo=T, message=F, warning = FALSE-----------
library(ggplot2) #version: ggplot2_3.4.4
library(ggridges) #version: ggridges_0.5.6
library(RColorBrewer) #version: RColorBrewer_1.1-3
library(dplyr) #version: dplyr_1.1.4
library(ggrepel) # version: ggrepel_0.9.5
library(reshape2) #version: reshape2_1.4.4


## Rename for interpretability:
## Mapping:
## Theta1-Culture threat
## Theta2-ReligionThreat
## Theta3-Economic Threat
## Theta4-HealthThreat
## Theta5-OutcomeSupportImmigration
## Theta6-OutcomeSupportEU

colnames(thetas)[1:6] <-  recode(colnames(thetas)[1:6],
                                 "Theta1" = "Culture Threat",
                                 "Theta2" = "Religion Threat",
                                 "Theta3" = "Economic Threat",
                                 "Theta4" = "Health Threat",
                                 "Theta5" = "Support Immigration",
                                 "Theta6" = "Support EU")

#Save aggregate plot
ggbase <- irt_vis(d = d, T_out = thetas, sub_name = NULL, out_file = "ebirtm-synth.png")


## ----medtrust-viz, eval=T, echo=T, message=F, warning = FALSE-----------------

#Save plot subset by media trust
ggmt <- irt_vis(d = d, T_out = thetas, sub_name = "mediatrust", out_file = "theta-media-synth.png")


## ----lambdas, eval=TRUE, echo=T, message=F, warning = FALSE-------------------

#Extract relevant substantive notes and create data frame with them and item codes
filtered_MCodes <- MCodes[MCodes[[2]] %in% M_matrix$QMap, , drop = FALSE]
M_df <- data.frame(QMap = M_matrix$QMap, sn = filtered_MCodes[[3]])

#Explore item loadings
lambdas <- get_lambdas(irt$lambda, item_names = M_df$QMap, dim_names = theta_names, item_elab = M_df$sn)
average_lambdas <- lambdas[[1]]
highest_lambdas <- lambdas[[2]]
average_lambdas
highest_lambdas

### ----------------------------
### Test: continuous IRT-M model
### ----------------------------

set.seed(6889)

# Simulate a simple continuous-measurement model
N <- 100   # respondents
K <- 3     # items
d <- 1     # latent dimension

# True latent trait
theta_true <- rnorm(N, 0, 1)

# True loadings + intercepts
lambda_true <- matrix(c(0.8, -1.2, 0.5), ncol = 1)
b_true <- c(0.4, -0.2, 1.0)

# Generate continuous outcomes
Y <- matrix(NA, N, K)
for (k in 1:K) {
  mu <- theta_true * lambda_true[k] - b_true[k]
  Y[, k] <- rnorm(N, mean = mu, sd = 1)
}

# Run continuous IRTM
res <- M_constrained_irt_continuous(
  Y,
  d = d,
  nburn = 2000,
  nsamp = 4000,
  thin = 5,
  learn_Sigma = TRUE,
  learn_Omega = TRUE,
  display_progress = FALSE
)

# Examine results
str(res)

# Posterior mean estimates
theta_est <- apply(res$theta, c(1, 2), mean)
lambda_est <- apply(res$lambda, c(1, 2), mean)
b_est <- rowMeans(res$b)

# Print comparison
cat("True Lambda:\n")
print(lambda_true)

cat("\nEstimated Lambda:\n")
print(lambda_est)

cat("\nTrue b:\n")
print(b_true)

cat("\nEstimated b:\n")
print(b_est)

# Simple correlation check on theta recovery
cat("\nCorrelation between true and estimated theta:\n")
print(cor(theta_true, theta_est))

#' @title pair_gen_anchors
#' @description This function generates anchor points from the M matrices.
#' It creates d(d-1)*4 fake respondents such that, for every pair of dimensions:
#' The first respondent has an extremely positive value of both dimensions of the pair,
#' The second has an extremely positive value for dim 1 and an extremely negative value for dim 2 of each pair,
#' The third has an extremely negative value for dim 1 and an extremely positive value for dim 2 of each pair,
#' The fourth has an extremely negative value of both dimensions of the pair.
#' These respondents' answers are imputed according to the directions of loadings specified by the M-matrices, i.e.,
#' if question k loads positively on dim 1 and positively on dim 2, the first respondent in the dim 1/dim 2 pair will have
#' yes for question k. If question k+1 loads  positively on dim 1 and negatively on dim 2 then the second respondent in
#' the dim 1/dim 2 pair will have yes on question k+1 and so on.
#' @param M a list containing K dxd M-matrices
#' @param A What value should be considered extreme for the latent dimensions.
#' @return A list with two elements:
#'   \item{Yfake}{A matrix of dimension (d(d-1)*4 x K) containing the imputed answers of the fake anchor respondents, where d is the number of dimensions and K is the number of questions. Values are 0 (no) or 1 (yes).}
#'   \item{theta_fake}{A list of d(d-1)*4 vectors, each of length d, representing the latent trait values for the fake anchor respondents. Each vector contains mostly zeros, with extreme values (A or -A) at the positions corresponding to the pair of dimensions being considered.}
#' @export
#' @export

pair_gen_anchors = function(M,A){
  K = dim(M)[3]
  d = ncol(M)
  M[is.na(M)]=9
  if (d>1) {
    Yfake = matrix (NA, d*(d-1)*4, K)
    theta_fake = list()
    i = 1
    for (j in 1:d){
      for (l in 1:d){
        if(j==l)
          next
        for(k in 1:K){
          #YY
          Yfake[i, k] = ifelse((M[j, j, k] == 1 && M[l, l, k] != -1) ||
                                 (M[l, l, k] == 1 && M[j, j, k] != -1), 1, NA)
          if(is.na(Yfake[i, k]))
            Yfake[i, k] = ifelse((M[j, j, k] == -1 && M[l, l, k] != 1) ||
                                   (M[l, l, k] == -1 && M[j, j, k] != 1), 0, NA)

          #NY
          Yfake[i+1, k] = ifelse((M[j, j, k] == -1 && M[l, l, k] != -1) ||
                                   (M[l, l, k] == 1 && M[j, j, k] != 1), 1, NA)
          if(is.na(Yfake[i+1, k]))
            Yfake[i+1, k] = ifelse((M[j, j, k] == 1 && M[l, l, k] != 1) ||
                                     (M[l, l, k] == -1 && M[j, j, k] != -1), 0, NA)

          #YN
          Yfake[i+2, k] = ifelse((M[j, j, k] == 1 && M[l, l, k] != 1) ||
                                   (M[l, l, k] == -1 && M[j, j, k] != -1), 1, NA)
          if(is.na(Yfake[i+2, k]))
            Yfake[i+2, k] = ifelse((M[j, j, k] == -1 && M[l, l, k] != -1) ||
                                     (M[l, l, k] == 1 && M[j, j, k] != 1), 0, NA)

          #NN
          Yfake[i+3, k] = ifelse((M[j, j, k] == -1 && M[l, l, k] != 1) ||
                                   (M[l, l, k] == -1 && M[j, j, k] != 1), 1, NA)
          if(is.na(Yfake[i+3, k]))
            Yfake[i+3, k] = ifelse((M[j, j, k] == 1 && M[l, l, k] != -1) ||
                                     (M[l, l, k] == 1 && M[j, j, k] != -1), 0, NA)

        }
        theta_fake[[i]] = rep(0, d)
        theta_fake[[i]][j] = A
        theta_fake[[i]][l] = A
        theta_fake[[i+1]] = rep(0, d)
        theta_fake[[i+1]][j] = -A
        theta_fake[[i+1]][l] = A
        theta_fake[[i+2]] = rep(0, d)
        theta_fake[[i+2]][j] = A
        theta_fake[[i+2]][l] = -A
        theta_fake[[i+3]] = rep(0, d)
        theta_fake[[i+3]][j] = -A
        theta_fake[[i+3]][l] = -A
        i = i+4
      }
    }
  }else if (d==1) {
    Yfake = matrix (NA, 2, K)
    theta_fake = list()
    for(k in 1:K){
      #Y
      if(M[1, 1, k] == 1) {
        Yfake[1, k]=1
      }else if(M[1, 1, k] == -1) {
        Yfake[1, k]=0
      }
      #N
      if(M[1, 1, k] == -1) {
        Yfake[2, k]=1
      }else if(M[1, 1, k] == 1) {
        Yfake[2, k]=0
      }
    }
    theta_fake[[1]] = rep(0, d)
    theta_fake[[1]][1] = A
    theta_fake[[2]] = rep(0, d)
    theta_fake[[2]][1] = -A
  }else {
    stop("The number of dimensions is less than 1.")
  }
  fakeList<- list("Yfake" = Yfake, "theta_fake" = theta_fake)
  return(fakeList)
}

#' @title irt_m
#' @description This function is a wrapper to enable easier use of the IRT-M model in M_constrained_irt.
#' It takes as input two data frames: a N x K data frame, and a K x (1+d) M-matrix.
#' The first column of the M-matrix should contain item identifiers that match the K column headers in
#' the N x K data frame. If they do not match, the wrapper exits with an error.
#' The wrapper computes anchors, Y_all (merged data and anchors), and a list of diagonal M-Matrices.
#' The second two are used as inputs to M_constrained_irt, which runs the sampler. Also used as input are
#' nburn (Default = 10^3), nsamp (Default = 10^3), thin (Default = 1), and
#' learn_loadings (Default = FALSE). This last one defaults to having the sampler learn factor covariances.
#' If set to true, it will learn loading covariances instead.
#' Finally, the wrapper removes the anchors and returns an irt list.
#' @param Y_in a N x K matrix of responses given by N respondents to K items. Can contain missing values. Column names should match first column in M_matrix.
#' @param d an integer specifying the number of latent dimensions.
#' @param M_matrix a K x (d+1) matrix of theoretical codings used to constrain IRT-M  (default=NULL). First column should match column names in Y_in.
#' @param nburn an integer specifying the number of burn-in MCMC iterations.
#' @param nsamp an integer specifying the number of sampling MCMC iterations.
#' @param thin an integer specifying the number of thinning MCMC samples.
#' @param learn_loadings a Boolean specifying whether a covariance matrix for the latent loadings should be learned, instead of the default covariance matrix for latent dimensions.
#' @return A list containing the following components:
#'   \item{lambda}{An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.}
#'   \item{b}{A matrix of dimension (K x nsamp/thin) containing posterior samples of item difficulty parameters.}
#'   \item{theta}{An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values.}
#'   \item{Sigma}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of latent traits (only if learn_Sigma=TRUE).}
#'   \item{Omega}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of item loadings (only if learn_Omega=TRUE).}
#' @import RcppProgress
#' @importFrom tmvtnorm rtmvnorm
#' @importFrom truncnorm rtruncnorm
#' @importFrom RcppArmadillo armadillo_version
#' @importFrom RcppDist bayeslm
#' @export

## usethis namespace: start
#' @useDynLib IRTM, .registration = TRUE
#' @importFrom Rcpp sourceCpp
## usethis namespace: end

irt_m = function(Y_in, d, M_matrix=NULL,
                 nburn=1000, nsamp=1000, thin=1,
                 learn_loadings=FALSE){

  d_which_fix = NULL
  d_theta_fix = NULL
  if (!is.null(M_matrix)) {
      #First check to ensure that the K column headers in the input data Y_in
      #match the elements in the first column of the constraint matrix M_matrix
      #exit if not
      if(length(setdiff(colnames(Y_in), M_matrix[,1]))!=0) {
        stop("Item labels do not match. Please address inconsistency before running again.")
      }
      #Next check to ensure that M has d+1 dimensions if not NULL.
      if (!is.null(M_matrix) && dim(M_matrix)[2]!=(d+1)) {
        stop("The number of latent dimensions does not match the number coded in M_matrix.")
      }

    #Create array of M-matrices
    M <- array(NA, c(d, d, ncol(Y_in)))
    #if only one dimension, force a matrix to be constructed
    if (d==1) {
      for (i in 1:ncol(Y_in)) {
        M[,,i] <- matrix(M_matrix[i,2], nrow = 1, ncol = 1)
      }
    }
    else {
      codings <- 2:(d+1)
      for (i in 1:ncol(Y_in)) {
        M[,,i] <- diag(M_matrix[i,codings])
      }
    }

    #Check dimensions of created objects and exit if incorrect for sampler
    if(dim(Y_in)[2] != dim(M)[3]) {
      stop("Dimensions of Data and Constraint do not match.")
    }
    if(dim(M)[1] != d || dim(M)[2]!=d) {
      stop("M is not dxd; check processing")
    }

    #First, compute synthetic anchor points using IRT-M's built-in function
    #The value of 5 is chosen to be an extreme value, but 5 is arbitrary
    l2 <- pair_gen_anchors(M,5)
    Yfake <- l2[[1]]
    colnames(Yfake) <- names(Y_in)
    theta_fake <- l2[[2]]
    #Second, create a data matrix for the sampler that contains the fake data as well.
    Y_all = as.matrix(rbind(Yfake, Y_in))
    d_which_fix <- 1:nrow(Yfake)
    d_theta_fix <- theta_fake
  }

  #Run constrained_IRT
  #Default to learning factor covariance unless user sets learn_loadings to TRUE.
  #Note: these ae mutually exclusive: one can't learn both.
  #Setting both learn_Sigma and learn_Omega to TRUE defaults to learning factor covariance.
  learnS <- TRUE
  learnO <- FALSE
  if (learn_loadings==TRUE) {
    learnS <- FALSE
    learnO <- TRUE
  }

  #Run IRT-M
  irt <- M_constrained_irt(Y_all,
                           d = d,
                           M= M,
                           theta_fix = d_theta_fix,
                           which_fix = d_which_fix,
                           nburn=nburn,
                           nsamp=nsamp,
                           thin=thin,
                           learn_Sigma=learnS, learn_Omega=learnO)

  if (!is.null(Yfake)) {
    #Remove anchors before returning results
    anchors_end <- dim(Yfake)[1]
    irt$theta <- irt$theta[-(1:anchors_end), , , drop=FALSE]
  }
  return(irt)
}

library(tidyverse) # version: tidyverse_2.0.0
library(dplyr) #version: dplyr_1.1.4
library(stats) # version: stats4
library(fastDummies) # version: fastDummies_1.7.3
library(reshape2) #version: reshape2_1.4.4
library(testthat)

## IRT-M estimation:
#devtools::install_github("dasiegel/IRT-M")
library(IRTM) #version

test_that("Test Case One: Normal Use", {

  synth_questions <- NULL  # Initialize to avoid R CMD check notes

  load("synth_questions.rda")
  ## Convert numeric ordinal responses to factors

  ebdatsub <- lapply(ebdatsynth[,], factor) ## that's a list now

  ## converts the list back into a dataframe:
  Y <- dummy_cols(.data=ebdatsub,
                  remove_selected_columns=TRUE)

  ## remove the .data that dummy_cols adds to the column names
  colnames(Y) <- gsub(".data.", '', colnames(Y))

  ## remove the data objects:
  rm(ebdatsub)
  rm(ebdatsynth)

  load('mcodes.rda')

  ## Only keep M-Codes with loadings or outcomes:
  MCodes$encoding <- rowSums(abs(MCodes[,4:9]))
  MCodes <- MCodes[which(MCodes$encoding > 0),]

  d <- 6 #number of coded dimensions
  mcolumns <- c("QMap", "D1-Culture threat",
                "D2-ReligionThreat",
                "D3-Economic Threat",
                "D4-HealthThreat",
                "O1-OutcomeSupportImmigration", "O2-OutcomeSupportEU")

  combine <- MCodes[,mcolumns] %>% ## question codes and loadings
    inner_join(
      Y %>%
        t() %>%
        as.data.frame(stringsAsFactors = FALSE) %>%
        rownames_to_column(var = "question"),
      by = c("QMap" = "question"  )
    )

  M_matrix <- as.data.frame(combine[, 1:(d+1)])

  #Reverse the earlier transposition of the observations:
  Y_in <- combine[, (d+2):ncol(combine)]%>%
    t() %>%
    as.data.frame()

  Y_in <- as.data.frame(sapply(Y_in, as.numeric))

  ## Take the question names and
  ## convert to column names

  question <- combine[,1] %>%
    as.data.frame()
  colnames(Y_in) <- question[,1]
  rm(combine)
  rm(question)

  #Run IRT-M
  d=6
  irt <- irt_m(Y_in = Y_in,
               d = d,
               M_matrix = M_matrix,
               nsamp = 100,
               nburn=100, thin=1)

  ## Check outputs:
  expect_type(irt, "list") ## outputs are a list

   ## Verify components:
  expect_named(
    irt,
    c("theta", "lambda", "b",
      "Sigma", "Omega",
      "hyperparameters", "constraints")
  )

  ## Verify expected shape of important results:
  expect_true(is.array(irt$theta)) #theta is an array
  expect_true(is.array(irt$lambda)) #lambda is an array
}) ## closes the test_that() call

#' @title Mean Squared Error
#' @description loss function for benchmarks
#' @param ytrue observed values,
#' @param ypred predicted values
#' @param aggregate logical for whether to take mean of estimate
#' @param root logical for whether to return square root of MSE
#' @return mean squared error
#' @importFrom stats sd
#' @export

mse <- function(ytrue, ypred, aggregate=TRUE, root=FALSE){
  ytrue_sd <- apply(ytrue, 2, sd)
  ypred_sd <- apply(ypred, 2, sd)
  if (aggregate)
    ret <- colMeans((ytrue/ytrue_sd-ypred/ypred_sd)^2)
  else
    ret <- (ytrue/ytrue_sd-ypred/ypred_sd)^2
  if (root)
    ret <- sqrt(ret)

  return(ret)
}

#' @title Geweke Convergence
#' @description Runs Geweke tests to assess MCMC convergence
#' @param THETA Matrix of parameter estimates from IRTM
#' @return Proportion of values that fail the Geweke convergence test (p < 0.05) for each parameter
#' @importFrom coda geweke.diag
#' @importFrom stats pnorm
#' @export

Geweke_convergence <- function(THETA){
  N <- dim(THETA)[1]
  d <- dim(THETA)[2]
  count <- c(0,0,0)
  P <- matrix(NA, N-6, 3)
  for(i in 7:N){
    for(j in 1:3){
      z <- geweke.diag(THETA[i,j,], frac1=0.1, frac2=0.5)
      p <- 2*pnorm(-abs(z$z))
      P[i-6,j] <- p
      if(p < 0.05){
        count[j] = count[j] + 1
      }
    }
  }
  count/N
}

#' @title Standardize Theta
#' @description standardizes theta estimates
#' @param theta estimated object
#' @param Sigma covariance matrix
#' @return theta divided by sigma param
#' @export

standardize_theta <- function(theta, Sigma){
  theta_std <- t(t(theta)/sqrt(diag(Sigma)))
  theta_std
}

#' @title Theta Lambda Traceplots
#' @description Creates traceplots for IRT parameter convergence diagnostics
#' @param irt An object containing theta and lambda parameters from an IRTM model
#' @param i Index of the respondent to plot (randomly selected if NULL)
#' @param k Index of the item to plot (randomly selected if NULL)
#' @return Plots of theta, lambda, and their product across MCMC iterations
#' @importFrom graphics par
#' @export

theta_lambda_traceplots <- function(irt, i=NULL, k=NULL){

  # Store original par settings
  oldpar <- par(no.readonly = TRUE)
  # Restore original when function exits
  on.exit(par(oldpar))


  theta = irt$theta
  lambda = irt$lambda
  if (is.null(i))
    i = sample(1:nrow(theta), 1)
  if (is.null(k))
    k = sample(1:nrow(lambda), 1)
  n = length(theta[i, 1, ])
  D = ncol(theta)
  par(mfcol=c(D, 3))
  for(d in 1:D){
    plot(1:n, theta[i, d, ], xlab = 'sample', ylab=paste('theta', d))
  }
  for(d in 1:D){
    plot(1:n, lambda[k, d, ], xlab = 'sample', ylab=paste('lambda', d))
  }
  for(d in 1:D){
    plot(1:n, theta[i, d, ] * lambda[k, d, ], xlab = 'sample', ylab=paste('theta * lambda', d))
  }
}

#' @title Average Thetas
#' @description Compute matrix of theta means over posterior distributions
#' @param theta_array An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values
#' @return theta_av N x d matrix of average thetas
#' @export
#'

theta_av <- function(theta_array) {

  N <- dim(theta_array)[1]
  d <- dim(theta_array)[2]
  z <- dim(theta_array)[3]

  # Reshape the array to 2D: (N * d) x nsamp/thin
  reshaped_array <- matrix(theta_array, nrow = N * d, ncol = z)

  # Compute the mean for each element and reshape back to N x d
  tav <- matrix(rowMeans(reshaped_array), nrow = N, ncol = d)
  rm(reshaped_array)
  rm(N)
  rm(d)
  rm(z)

  return(tav)
}

#' @title irt_vis
#' @description Takes as input the number of latent dimensions (d),
#' an N x (d+z) data frame with average thetas in the first d columns and variables not included in the
#' calculation of the thetas in the rest (T_out), and, optionally, a variable name (sub_name)
#' taken from T_out, and an output file name (out_file),
#' and returns either unconditional theta distributions or distributions subset by that variable
#' @param d The number of latent dimensions
#' @param T_out N x (d+z) data frame with average latent dimensions in first d columns
#' @param sub_name The name of a variable in T_out used for levels in the plot (Default = NULL)
#' @param out_file Output file name for plot (Default = NULL)
#' @return A ggplot2 object containing density ridge plots of the latent dimensions. When sub_name is NULL, the plot shows the distribution of each theta dimension. When sub_name is provided, the plot shows distributions faceted by theta dimension and grouped by the specified variable.
#' @importFrom reshape2 melt
#' @import ggplot2
#' @importFrom dplyr mutate
#' @importFrom ggridges geom_density_ridges
#' @importFrom rlang sym
#' @export
#'

irt_vis <- function(d, T_out, sub_name=NULL, out_file=NULL) {

  #initialize plot output
  irtv <- NULL
  theta_names <- colnames(T_out[1:d])
  Value<-NULL
  Theta<-NULL
    if (is.null(sub_name)) {   #If desired output is aggregated
    basedata <- melt(T_out[,1:d])
    if (d==1) { #When there is only one latent dimension
      colnames(basedata) <- c("Value")
      irtv <- ggplot(mutate(basedata,Theta = theta_names[1]),  # Replace Theta with a constant value
                     aes(x = Value,
                         y = Theta,
                         fill = Theta)) +
        geom_density_ridges() +
        scale_y_discrete(limits = rev(levels(factor(theta_names[1])))) +
        ggtitle('Posterior Distribution of Group Mean',
                subtitle = theta_names[1]) +
        scale_fill_brewer(palette = 'Spectral') +
        theme_bw()
    } else if (d>1) { #If there is more than one latent dimension
      colnames(basedata) <- c("Theta", "Value")
      irtv <- ggplot(basedata,
                         aes(x=Value,
                             y=Theta,
                             fill=Theta))+
        geom_density_ridges() +
        scale_y_discrete(limits = rev(levels(basedata$Theta))) +
        ggtitle('Posterior Distribution of Group Mean',
                subtitle= "Thetas") +
        scale_fill_brewer(palette='Spectral') +
        theme_bw()
    } else {
      stop("You have less than one latent dimension specified.")
    }
  } else { #Output is disaggregated by variable sub_name, which should be a factor

      mt_cols <- c(theta_names,sub_name)
      dat2 <- melt(T_out[,mt_cols])
      colnames(dat2) <- c(sub_name, "Theta", "Value")
      irtv <- ggplot(dat2,
                          aes(x=Value,
                              y=!!sym(sub_name),
                              fill=!!sym(sub_name)))+
        geom_density_ridges(alpha=.5) +
        ggtitle('Posterior Distribution of Group Mean',
                subtitle= "Thetas") +
        labs(y=sub_name,
             x= "Theta Posterior Estimates")+
        scale_y_discrete(limits=rev(levels(dat2$sub_name)))+
        facet_wrap(~Theta, ncol=2)+
        scale_fill_brewer(palette='Spectral') +
        theme_bw()
    }

  #Print plot to file if desired by user
  if (!is.null(out_file)) {
    ggsave(irtv, filename=out_file, width=5, height=5, units ="in", dpi=300)
  }
  return(irtv)
}

#' @title get_lambdas
#' @description Takes as input the array of lambdas from the irt list,
#' a vector of item names (can be taken from either Y_in or M_matrix), a vector of dimension names,
#' and, optionally, a vector comprising elaborations about each item.
#' Returns a list containing a data frame with the mean lambdas for each item-dimension pair,
#' possibly attaching elaborations to each item's string,
#' and a data frame with the items with the highest mean values of lambda for each dimension in order
#' @param lambda_array An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.
#' @param item_names Vector of item names.
#' @param dim_names Vector of dimension names.
#' @param item_elab A vector comprising elaborations about each item (Default = NULL).
#' @return A list containing the following components:
#'   \item{av_lams}{A data frame of dimension (K x (1+d)) containing averages of item discrimination parameters.}
#'   \item{high_lams}{A data frame of dimension (K x d) containing an ordered list of the items with the highest mean values of lambda for each dimension.}
#' @export
#'

get_lambdas <- function(lambda_array, item_names, dim_names, item_elab=NULL) {

  #initialize output list
  lambdas <- vector("list", length=2)

  #Find average lambdas
  K <- dim(lambda_array)[1]
  d <- dim(lambda_array)[2]
  z <- dim(lambda_array)[3]
  # Reshape the array to 2D: (K * d) x nsamp/thin
  reshaped_array <- matrix(lambda_array, nrow = K * d, ncol = z)
  # Compute the mean for each element and reshape back to N x d
  lav <- matrix(rowMeans(reshaped_array), nrow = K, ncol = d)
  rm(reshaped_array)
  rm(K)
  rm(d)
  rm(z)

  #Combines elaborations into item strings if user supplied
  inames <-item_names
  if (!is.null(item_elab)) inames <- paste(item_names, item_elab, sep=": ")

  #Creates data frame for output of average lambdas
  av_lambs <- as.data.frame(cbind(inames, lav))
  colnames(av_lambs)<-c("QMap",dim_names)
  av_lambs[dim_names]<-lapply(av_lambs[dim_names], as.numeric)
  lambdas[[1]]<-av_lambs

  # Create data matrix with descending order
  # Extract the first column from av_lambs
  values <- av_lambs[[1]]
  # Extract the ordering columns (last d columns) and take their absolute value
  order_columns <- av_lambs[ , -1, drop=FALSE]
  order_columns <- as.data.frame(lapply(order_columns, abs))

  # Number of ordering columns (d)
  d <- ncol(order_columns)
  # Initialize an empty list to store reordered columns
  reordered_columns <- vector("list", d)

  # Loop through each ordering column and reorder `values` in descending order
  for (i in seq_len(d)) {
    reordered_columns[[i]] <- values[order(-order_columns[[i]])]
  }

  # Combine reordered columns into a data frame
  high_lambs <- as.data.frame(reordered_columns)
  # Set column names for B
  colnames(high_lambs) <- dim_names

  lambdas[[2]]<-high_lambs

  return(lambdas)
}

#' @title learned correlations
#' @description Takes as input either the Sigma covariance matrix, if the user has learned the factor covariance,
#' or the Omega covariance matrix, if the user has learned the loading covariance, as well as a vector of
#' dimension names.
#' Returns a correlation matrix with correlations between the dimensions.
#' @param cov_array An array of dimension (d x d x nsamp/thin) containing posterior samples of the relevant covariance matrix.
#' @param dim_names Vector of dimension names.
#' @return A data frame containing the correlation matrix derived from t input covariance array, with rows and columns labeled according to dim_names (if provided). Each cell represents the correlation between the corresponding dimensions.
#' @export
#'

dim_corr <- function(cov_array, dim_names = NULL) {

  d <- dim(cov_array)[1]
  z <- dim(cov_array)[3]

  # Reshape the array to 2D: (d * d) x nsamp/thin
  reshaped_array <- matrix(cov_array, nrow = d * d, ncol = z)

  # Compute the mean for each element and reshape back to d x d
  covav <- matrix(rowMeans(reshaped_array), nrow = d, ncol = d)
  rm(reshaped_array)
  rm(z)

  #compute correlation matrix
  # Compute the standard deviations (sqrt of diagonal elements)
  std_devs <- sqrt(diag(covav))

  # Create the correlation matrix
  corav <- covav / (std_devs %*% t(std_devs))

  corav<-as.data.frame(corav)

  if (!is.null(dim_names)) {
    if (length(dim_names)==d) {
      colnames(corav)<-dim_names
      rownames(corav)<-dim_names
    }
  }

  return(corav)
}

#' Datasets for IRT-M Package
#'
#' MCodes, synth_idvs, and synth_questions are included in the vignette
#'
#' @name MCodes
#' @docType data
#' @title Methodological Codes
#' @description
#' Factor loading matrix for IRT-M vignette. This is a 793 row and 9 column dataset.
#' The rows are derived from the binary encoding of the synthetic survey, with a row for every
#' binarized question in the survey. The first 56 rows are retained metadata, and have lots of NA.
#'
#' The data format is an intermediary processing for IRT-M, and is detailed in the vignette text.
#'
#'  @format A data frame with the following variables:
#'#' \describe{
#'   \item{QCode}{Mapping of the dimension coding key to the underlying question in the original (synthetic) survey data.
#'    The first 56 rows are blank because they map to survey and respondent metadata that doesn't relate to dimensions.}
#'   \item{QMap}{Mapping to the question in data processed by the vignette for variable estimation. }
#'   \item{SubstantiveNotes}{Brief human readable comments on the substantive meaning of the coded questions. These are for convenience of reference.}
#'   \item{D1-Culture threat}{Loading vector for the cultural threat dimension. A 1 indicates that the question is expected to load.}
#'   \item{D2-ReligionThreat}{Loading vector for the religious threat dimension.}
#'   \item{D3-Economic Threat}{Loading vector for the economic threat dimension.}
#'   \item{D4-HealthThreat}{Loading vector for the health threat dimension.}
#'   \item{O1-OutcomeSupportImmigration}{Loading vector for the immigration support composite.}
#'   \item{O2-OutcomeSupportEU}{Loading vector for the European Union support composite.}
#' }

#' @source
#' IRT-M vignette walk through.
#'
#' @keywords datasets
NULL

#' @name synth_idvs
#' @docType data
#' @title Synthetic Independent Variables
#' @description
#' A synthetic dataset of independent variables for post-estimate analysis in the vignette.
#' Extraction of the data from the synthetic survey is described in the vignette.
#'
#' @format A 3000 row and 27 column dataset of synthetic survey responses. This closely follows the 94.3 Eurobarometer survey in structure.
#' This dataset is a toy that is intended for the IRT-M vignette. For real analysis, see the original Eurobarometer data collection.
#'

#' @keywords datasets
NULL

#' @name synth_questions
#' @docType data
#' @title Questions for the Synthetic European sentiment survey in the vignette
#' @description
#' A synthetic dataset with 3000 rows and 148 questions.
#' This data replicates the structure of questions following Eurobarometer 94.3 (2021).
#' It is not intended to be analyzed independently from the vignette.
#' '
#' @format A data frame with 3000 rows and 148 columns representing synthetic survey responses
#'
#' @keywords datasets
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sample_constrained_irt <- function(Y, d, nu0, S0, lbs, ubs, ind, theta_fix, nburn, nsamp, thin, learn_Sigma, learn_Omega, display_progress) {
    .Call(`_IRTM_sample_constrained_irt`, Y, d, nu0, S0, lbs, ubs, ind, theta_fix, nburn, nsamp, thin, learn_Sigma, learn_Omega, display_progress)
}

#' @title M_constrained_irt
#' @description This function allows you to run the IRT model.
#' @param Y a N x K matrix of responses given by N respondents to K items. Can contain missing values.
#' @param d an integer specifying the number of latent dimensions.
#' @param M a list of K d x d matrices (default=NULL).
#' @param theta_fix a matrix with d columns containing the values of the latent dimensions for respondents that have pre-specified latent factors.
#' @param which_fix a vector containing the indices of the respondents for which latent factors have been fixed.
#' @param nburn an integer specifying the number of burn-in MCMC iterations.
#' @param nsamp an integer specifying the number of sampling MCMC iterations.
#' @param thin an integer specifying the number of thinning MCMC samples.
#' @param learn_Sigma a Boolean specifying whether a covariance matrix for the latent factors should be learned.
#' @param learn_Omega a Boolean specifying whether a covariance matrix for the latent loadings should be learned.
#' @param hyperparameters a list of hyperparameters for the model.
#' @param display_progress a Boolean specifying whether a progress bar should be displayed.
#' @return A list containing the following components:
#'   \item{lambda}{An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.}
#'   \item{b}{A matrix of dimension (K x nsamp/thin) containing posterior samples of item difficulty parameters.}
#'   \item{theta}{An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values.}
#'   \item{Sigma}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of latent traits (only if learn_Sigma=TRUE).}
#'   \item{Omega}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of item loadings (only if learn_Omega=TRUE).}
#' @import RcppProgress
#' @importFrom tmvtnorm rtmvnorm
#' @importFrom truncnorm rtruncnorm
#' @importFrom RcppArmadillo armadillo_version
#' @importFrom RcppDist bayeslm
#' @export

## usethis namespace: start
#' @useDynLib IRTM, .registration = TRUE
#' @importFrom Rcpp sourceCpp
## usethis namespace: end

M_constrained_irt = function(Y, d, M=NULL, theta_fix=NULL, which_fix=NULL,
                             nburn=1000, nsamp=1000, thin=10,
                             learn_Sigma=TRUE, learn_Omega=FALSE,
                             hyperparameters = list(),
                             display_progress=TRUE){

  N = nrow(Y)
  K = ncol(Y)
  S = nsamp + nburn

  if(is.null(hyperparameters[['S0']]))
    S0 = diag(d)
  else
    S0 = hyperparameters[['S0']]

  if(is.null(hyperparameters[['O0']]))
    O0 = diag(d) * 25 # same as pscl default
  else
    O0 = hyperparameters[['O0']]

  if(is.null(hyperparameters[['nu0']]))
    nu0 = d
  else
    nu0 = hyperparameters[['nu0']]

  if(is.null(hyperparameters[['mu']]))
    mu = rep(0, d)
  else
    mu = hyperparameters[['mu']]

  #This is here in case one wants to pass a NULL M directly to M_constrained_irt rather than going through irt_m.
  if(is.null(M)){
    M = array(NA, c(d, d, K))
    for(k in 1:K)
      ## 2 used as a signal value unconstrained diagonal entries when M is NULL
      ## the diag *2 intended to kick into unconstrained mode
      M[,,k] = diag(d)*2

  }

  lbs = matrix(NA_real_, K, d)
  ubs = matrix(NA_real_, K, d)
  for (k in 1:K){
    for (j in 1:d){
      val <- M[j,j,k]
      if(is.na(val)){
        lbs[k, j] = -Inf
        ubs[k, j] = Inf
      }else if(val == 0){
        lbs[k, j] = 0
        ubs[k, j] = 0
      }else if (val == 1){
        lbs[k, j] = 0
        ubs[k, j] = Inf
      }else if (val == -1){
        lbs[k, j] = -Inf
        ubs[k, j] = 0
      }else{
        lbs[k, j] = -Inf
        ubs[k, j] = Inf
      }
    }
  }

  ## initialize
  if(is.null(theta_fix)){
    ind = numeric(0)
    theta_fix = numeric(0)
  }else{
    if(is.null(which_fix))
      ind = 1:length(theta_fix)
    else
      ind = which_fix
  }
  fixed_vals = matrix(unlist(theta_fix), length(theta_fix), d, byrow=T)

  if(display_progress)
    message('Sampling...')
  res = sample_constrained_irt(Y, d, nu0, S0, lbs, ubs, ind, fixed_vals,
                               nburn, nsamp, thin, learn_Sigma, learn_Omega, display_progress)

  res$hyperparameters <- list(S0=S0, O0=O0, nu0=nu0, mu=mu)
  res$constraints <- list(lbs=lbs, ubs=ubs)
  return(res)
}

---
title: "IRT-M Vignette (Synthetic Data)"
author: "Margaret J Foster"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{IRT-M Vignette (Synthetic Data)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
## Packages

Packages that we will use for data prep, IRT-M estimation, and data visualization:
```{r packages, eval=T, echo=T, message=F, warning = FALSE}
## Data prep:
library(tidyverse) # version: tidyverse_2.0.0
library(dplyr) #version: dplyr_1.1.4
library(stats) # version: stats4
library(fastDummies) # version: fastDummies_1.7.3
library(reshape2) #version: reshape2_1.4.4

## IRT-M estimation:
#devtools::install_github("dasiegel/IRT-M")
library(IRTM) #version 1.00

## Results visualization:
library(ggplot2)  # version: ggplot2_3.4.4
library(ggridges) #version: ggridges_0.5.6
library(RColorBrewer) #version: RColorBrewer_1.1-3
library(ggrepel) # version: ggrepel_0.9.5
```

In this document, we walk through using the IRT-M package.

The IRT-M framework may be used for a wide variety of cases. This vignette focuses on a single, hypothetical, use case, in which a research team seeks empirical support for a hypothesis: in this case, the threat-response hypothesis that anti-immigration attitudes in Europe are associated with perceptions of cultural, economic, and security threats. We illustrate data preparation, IRT-M estimation, visualization, and analysis with a small synthetic dataset (N=3000) based on the Eurobarometer 94.3 wave. The real data can be accessed via: \url{https://search.gesis.org/research_data/ZA7780?doi=10.4232/1.14076}.

This example illustrates one of the strengths of the IRT-M model: the (very common) situation in which researchers have a theoretical question and related data that does not directly address the substantive question of interest. In this case, we work through a research question derived from the literature on European attitudes towards immigration. The threat-response hypothesis is supported by the literature~\cite{kentmen2017anti}, but was not a specific focus of the 2020-2021 Eurobarometer wave. Consequentially, the survey did not directly ask questions about the three threat dimensions of interest.  However, the survey did contain several questions adjacent to threat perception.  We can use those questions to build what we call an M-Matrix (a matrix of constraints for the item discrimination loadings present in item response theory models), and estimate latent threat dimensions.

Regardless of the use case, there are two steps involved in preparing data for the IRT-M Package:

+ First, one needs to acquire and appropriately format a dataset comprising a series of items, the responses to which one believes are related to the set of underlying theoretical concepts that one wants to measure. One input of IRT-M is this `N x K` data frame of `N` responses to `K` items. Column names of the data frame should provide the names of the items. We'll refer to this data frame as `Y_in` in what follows.
+ Second, one must create a key that specifies the theoretical connection between each item in one's data and each underlying concept. A second input of IRT-M, when run with constraints, is a `K x (1+d)` data frame of `K` items and their pre-specified theoretical connections to each of `d` dimensions. Those connections must be provided in the last d columns; the first column should contain item names, which must match the column names of the data matrix. We'll refer to this data frame as `M_matrix` in what follows.

## Loading the package; Dependencies

Gfortran is a necessary to load the package. Gfortran can be readily downloaded and the version installed can be checked. In Windows, enter into the command line "$ gfortran --version GNU Fortran." The easiest way to make sure this works in Windows is to install Rtools, making sure that its version matches the version of R you are running. On a Mac, enter into the terminal "which gfortran." Some machines running Mac OS encounter a well-known problem with R finding `gfortran`. The steps for solving this are documented online, and typically entail a combination of installing `Xcode`, using `homebrew` to install `gcc`, and/or installing `gfortran` directly from `Github`. After `gfortran` has been successfully installed, it is important to also have `GCC` (GNU Compiler Collection) installed. In addition to `gcc` allowing its installation, the IRT-M package depends on functions developed in other libraries, and the `Rfortran` dependency requires `gcc` to be installed.

## Formatting the Observed Data

Formatting the Input Data

Though we are working to extend it to other forms of data, at present, IRT-M operates only with binary (dichotomous) items. Thus, it requires that users convert continuous or categorical variables into a series of binary variables. Those binary variables will be the items, and responses to those items will be used to discern positions on each latent theoretical dimension one is attempting to measure.

The most straightforward way to convert to binary from a categorical variable is to use a library, such as `fastDummies`, to expand the entire set of variables into one-hot (binary) encoding. Similarly, one could use packages such as `arulesCBA` or `arules` to convert continuous variables to categorical variables, and then `fastDummies`to convert those categorical variables in binary variables. One's input data frame, Y_in, should have `N` dichotomous responses to each of `K` binary items; in addition to `0` and `1`, `NA` is also acceptable.

It is good practice, and will make one's life easier later, to capture the column headers of `Y_in` (which will be item codes) and export them for use in constructing the `M_matrix`. Doing so will avoid mismatches between the items in the input data and the items one will be coding.

The `Y_in` and `M_matrix` data frames described earlier are all one needs to use IRT-M. How one obtains them will vary by use case. We'll describe here how we obtained ours for this vignette, though some steps will be specific to our use case, in which we start with an M_matrix for the real, and complete, Eurobarometer 94.3 wave, but use input data that is a synthetic subset of the real data. (Morucci et. al 2024 used the real data.)

To begin, we reformat the input data so that each possible answer becomes a separate binary item (One Hot encoding). In preparing the data, we used the `dummy_cols()` utility from the `fastdummies` package. If running this exact code on your own system, please ensure that the `dataPath` variable is adjusted for your local file structure.

```{r load-dat, eval=T, echo=T, message=F, warning = FALSE}

synth_questions <- NULL  # Initialize to avoid R CMD check notes

load("./vdata/synth_questions.rda")
## Convert numeric ordinal responses to factors

ebdatsub <- lapply(ebdatsynth[,], factor) ## that's a list now

## converts the list back into a dataframe:
Y <- dummy_cols(.data=ebdatsub,
                       remove_selected_columns=TRUE)

## remove the .data that dummy_cols adds to the column names
colnames(Y) <- gsub(".data.", '', colnames(Y))

## remove the data objects:
rm(ebdatsub)
rm(ebdatsynth)
```

## Creating the M-Matrix

Creating the M_matrix

The core step in using the IRT-M package is to map the theoretical connections between items in the data and the underlying theoretical constructs of interest into an `M_Matrix` of dimension `K x (1+d)`. To do so, we go through every item in the data (here: every response to every question in the survey) and decide whether the item relates to zero, one, or more of our latent theoretical dimensions. For those items that we believe relate to a theoretical dimension of interest, we can code whether we expect it to positively load---meaning that higher values on that dimension are associated with greater responses to the item (here: greater means more likely to choose a given response to a given question)---or negatively load. We code positive "loading" as `1` for that item-dimension pair in the M_matrix, negative "loading" as `-1`. We can also denote that the item has no relationship with the theoretical dimension of interest, in which case we code `0` for that item-dimension pair. If we are unsure, we can use an `NA` value in the `M_matrix` for the item-dimension pair, and the model will assume relatively diffuse priors on that loading.

If one has followed this vignette's advice and exported a column of item codes when formatting the input data frame, `Y_in`, then the easiest way to specify the `M_matrix` is to open a spreadsheet with the first column (labeled `QMap` in the vignette) being the item codes, and the remaining `d` columns the theoretical dimensions to be measured (with appropriately named column headers). We have also found it to be worthwhile to insert a column with human-readable notes next to the item codes. This step adds some overhead, but---much like commenting code in general---is invaluable for debugging and analysis. You will need to remove that column before running the `M_matrix` through IRT-M, but there is also a spot in one of our analysis functions, described below, that can incorporate that information in your analysis.

For this vignette, we are interested whether respondents to the 2020-2021 Eurobarometer reported feeling social, cultural, and/or economic threats. Depending on the data, you may need to do some additional processing at this step to ensure that each question coded is a binary response that has a single relationship to each dimension being coded. Many surveys present questions formatted into feeling thermometers, matrices, or with many nested sub-questions. These may not load straightforwardly into a single dimension. For example, Question 377 (qa1a_4) in our data asks for respondents' feelings about various current situations, compactly presented via a feeling thermometer for several sub-questions. In this example we will code the fourth, “How would you judge the current situation in [….] your personal job situation.” The respondent’s answer is coded as a categorical variable ranging from 1-4, with a response of 1 corresponding to “Very Good” and 4 corresponding to “Very Bad.”  We imagine that this question directly relates to the “economic threat” underlying dimension that we hypothesize, with the low end of the scale indicating feelings of economic threat and the high end suggesting no threat.

Thus, we expand the levels of the question prompt to become four separate yes/no questions:

+ qa1a_4_1. Situation of personal job: very good.
+ qa1a_4_2 Situation of personal job: rather good
+ qa1a_4_3 Situation of personal job: rather bad
+ qa1a_4_4. Situation of personal job: very bad

Note that if you had followed our advice above, these items would have already been separated for you and would occur as rows of the spreadsheet you were using to specify your `M_matrix`. Also note that we do not have an omitted category in this coding. That is deliberate: even in a yes/no question, it is possible that responses of "yes" implicate latent theoretical dimensions differently from responses of "no". For example, a "yes" might suggest a particular action or thought process, but many actions or thought processes could be responsible for a "no".

Once the `M_matrix` contains a column of item codes for binary items, we can specify our theoretical loadings for those items. In the `M_matrix`, we add a `1` to the column for the Economic Threat dimension for qa1a_4_3 and qa1a_4_4. Substantively, this means that responses to the qa1a_4 with a value in the data of "3” or “4" are coded as individual respondents who likely would score high on the “economic threat” dimension.  We can also code the inverse and give qa1a_4_1 and qa1a_4_2 the value of `-1` in the `M_matrix` because we expect that survey respondents who answer that they feel that their personal job situation is “very good” or “rather good” likely would not score highly on economic threat.

We repeat that process for each of the items that we think relate in any fashion (positively, negatively, or in an unclear way) to any of our underlying theoretical dimensions. Note that an item may relate in some way to no dimensions, one dimension, or more than one dimension, and that these relationships may be dependent or independent. Also note that if you think an item may relate but are not sure how exactly, the safest option is an `NA`.

We can also use this same technique to discern latent outcome variables in the data, and include them among our latent dimensions. In other words, we can code items that relate to potential dependent variables as well, and have those dependent variables be additional dimensions in our `M_matrix`, as long as we are careful not to use the same items in measuring latent dimensions intended to be independent and dependent variables in the same correlational model later. This vignette additionally highlights the ability of the IRT-M model to handle inductive theoretical exploration. While coding the `M_matrix` for the three dimensions of threat noted above, we observed that the survey itself featured a number of questions about feelings of threat related to the ongoing Covid-19 pandemic. IRT-M easily supports inductive coding, and we added another dimension for the emergent category of "health threat."

The coding step is the only really time consuming step in the process, but it is what allows us to make theoretical claims about the substantive meaning of the latent dimensions. Fortunately, though the input data may have many items potentially to be coded, the actual number to code is likely to be substantially smaller than the total, as we only need to code those items that relate to the latent dimensions. We can ignore the rest, since the constrained IRT will not use any item which has loadings of 0 for every latent dimension.

For our vignette, we thus end up with a `K x (1+d)` matrix, where `K` is the number of theoretically-salient binarized questions and `d` is the combined number of underlying latent explanatory dimensions (4) and latent outcomes (2). For convenience, we processed the constraint coding in a separate table, which we import. We reduce its size by getting rid of rows of zeroes. The `MCodes` matrix that we important has question codes, item codes to match to the input data matrix, a column of human-readable comment, and then loadings for each item-dimension pair across six dimensions.

```{r load-mcodes, eval=T, echo=T, message=F, warning = F}
load('./vdata/mcodes.rda')

## Only keep M-Codes with loadings or outcomes:
MCodes$encoding <- rowSums(abs(MCodes[,4:9]))
MCodes <- MCodes[which(MCodes$encoding > 0),]

```

## Processing with IRT-M

The last step before we can start our analysis is to produce the `Y_in` and `M_matrix` data frames we need. To get these, we must ensure that the columns of `Y_in` (and their headers) match the rows of `M_matrix` (and the item codes in its first column). If we had followed our own advice, this would already be true, and if you have followed our advice, you can skip to the next step. Sometimes, however, one might have cause to code the constraint matrix before one hot encoding the input data, in which case some effort might be needed to ensure a match due to different column names produced by `dummy_cols`. In our case, as our data are synthetic and only a subset of the real data, we'll need to eliminate a bunch of items to ensure a match. We do that by merging the data and loadings, resulting in a `K x R` matrix where `K` is the number of binary questions that we have codes for and `R` is the number of instrument responses, before then reversing the combination. This is accomplished by the following code snippet:

```{r format-mcodes, eval=T, echo=T, message=F, warning = F}
## Produce a K-coded questions x R-responses data frame:

d <- 6 #number of coded dimensions
mcolumns <- c("QMap", "D1-Culture threat",
              "D2-ReligionThreat",
              "D3-Economic Threat",
              "D4-HealthThreat",
              "O1-OutcomeSupportImmigration", "O2-OutcomeSupportEU")

combine <- MCodes[,mcolumns] %>% ## question codes and loadings
    inner_join(
        Y %>%
        t() %>%
        as.data.frame(stringsAsFactors = FALSE) %>%
        type_convert() %>%
        rownames_to_column(var = "question"),
        by = c("QMap" = "question"  )
    )

M_matrix <- as.data.frame(combine[, 1:(d+1)])

#Reverse the earlier transposition of the observations:
Y_in <- combine[, (d+2):ncol(combine)]%>%
    t() %>%
    as.data.frame()

Y_in <- as.data.frame(sapply(Y_in, as.numeric))

## Take the question names and
## convert to column names

question <- combine[,1] %>%
    as.data.frame()
colnames(Y_in) <- question[,1]
rm(combine)
rm(question)
```

Troubleshooting note: it is important that the `Y_in` object be numeric and a matrix. You can check the data types by running `dplyr`’s `summary(response_object)` [or similar] and adjust the data type if needed. Likewise, if you run the IRT-M sampler and it returns the message `Error: Not compatible with requested type: [type=list; target=double]` you probably either passed a non-numeric data type in or didn't pass a matrix to the `Y_in` argument.

##Running IRT-M

Running IRT-M

Once we have the input data, `Y_in`, and the constraint matrix, `M_matrix`, we can now proceed to run IRT-M. The package provides a wrapper to make running the IRT-M sampler easier: the `irt_m()` function. It takes as arguments `Y_in` and `M_matrix`, as well as the number of latent dimensions, `d`. If no constraint matrix is provided, it runs an unconstrained IRT model. If `M_matrix` is provided and not `NULL`, the first two things `irt_m()` does is check to make sure that the column headers of `Y_in` match the item codes in the first column of `M_matrix`. It quits with an error if they do not. This is to ensure that the analyst is using the expected set of items in the computation. Then, `irt_m()` checks to make sure that the number of dimensions provided by the analyst (in `d`) is one fewer than the number of columns of `M_matrix`. After those checks, `irt_m()` instantiates a `K x d x d` array of diagonal matrices, built from `M_matrix`, that will be used for the sampler, and checks one more time to make sure that all the dimensions are correct, quitting with an error if not.

Next, `irt_m` creates a set of `4d(d-1)` (for `d>1`) or `2` (for `d=1`) synthetic anchor points via the package function `pair_gen_anchors()`. Those points are mathematically optional for the purposes of identifying the model, but are substantively helpful in that they aid in providing a clear substantive interpretation and consistent scale of the underlying space. For every pair of latent dimensions, four "fake" respondents are created. Those respondents are assumed to have each possible combination of maximum/minimum positions along each dimension in the pair. Then, the `M_matrix` is used to infer how those fake respondents would have responded to each item. For example, a respondent with maximal levels of economic and cultural threat would respond `1` to items that were coded positively relative to both latent dimensions and `0` to items that were coded negatively relative to both latent dimensions. `irt_m()` adds these fake respondents to `Y_in` and then passes this new matrix, `Y_all`, to the sampler, which returns a list with the estimated `Theta`, `Lambda`, `b`, `Sigma`, and `Omega` posterior distributions for the model. By default, the fake respondents are placed at the beginning of `Y_all`; however, before returning its list, `irt_m()` removes the fake respondents, so that their presence does not hinder further analyses. It does this via two objects that help to keep track of the introduced anchor points: `d_which_fix` and `d_theta_fix`. Those record the indices of the synthetic extreme points that were created in `pair_gen_anchors()`.

In addition to `Y_in`, `M_matrix`, and `d`, `irt_m()` also has four additional arguments that may be used if desired. Three are technical variables related to the sampler. These specify the number of burn-in MCMC iterations (`nburn`, `Default=1000`), the number of sampling MCMC iterations (`nsamp`, `Default=1000`), and the number of thinning MCMC samples (`thin`, `Default=10`). The fourth is a logical variable (`learn_loadings`, `Default=FALSE`). IRT-M defaults to learning the covariance matrix of the latent dimensions, but setting this variable to `TRUE` has it instead learn the covariance matrix of the item loadings.

One does not need to use `irt_m()` if one does not want: one can directly use the sampler instead via the `M_constrained_irt()` function. Doing so requires creating the array of constraint matrices and anchor points oneself, as those need to be passed to the sampler as arguments.

Moving to the output list of `irt_m()`, the `theta` array gives us samples from the posterior distribution of respondents' positions along each of the latent dimensions. This object is an array of dimension `N x d x nsamp/thin`, or number of respondents x number of latent dimensions x number of saved simulations. The `lambda` array, of dimension `K x d x nsamp/thin`,  contains samples from the posterior distribution of item loadings (discrimination parameters) for each item-dimension pair. The `b` output array, of dimension `K x nsamp/thin`,  contains samples from the posterior distribution of item difficulty parameters. The `Sigma` array, of dimension `d x d x nsamp/thin`,  contains samples from the posterior distribution of the covariance matrix of latent dimensions. It is only learned if `learn_loadings=FALSE`, which is the default. The `Omega` array, of dimension `d x d x nsamp/thin`,  contains samples from the posterior distribution of the covariance matrix of item loadings. It is only learned if `learn_loadings=TRUE`.

Note that in the following code we've reduced the sampling settings a bit from the default, since the goal of the vignette is merely to be instructive and running quickly (less than a minute) is helpful for that. One can also increase to `nsamp=10000` and `nburn=2000` if one desires and has the time, though using the default thin=10 (or greater) is recommended in that case. Running an unconstrained IRT, useful when the analyst wants to evaluate the impact of coding decisions, can be done by instead using `M_matrix=NULL` or simply leaving it out.

```{r run-irtm, eval=T, echo=T, message=F, warning = FALSE}
d=6
irt <- irt_m(Y_in = Y_in, d = d, M_matrix = M_matrix, nsamp = 1000, nburn=20, thin=1)

```

## Interpretation and Analysis

Interpretation and analysis

We include samples from the posterior distribution as the analyst may desire to use them in capturing measurement uncertainty. For our purely descriptive purposes here, however, we'll want to take the means of the relevant posterior distributions. The most important mean we'll take, in order to produce a population-level distribution of positions on the latent dimensions, is that of the `theta` array. In `helpers_analysis.R`, we provide a function, `theta_av()`, to do this. It takes as input the `theta` array and returns an `N x d` dimensional matrix of estimated positions for each respondent, for each dimension.

```{r theta-average, eval=T, echo=T, message=F, warning = FALSE}
avgthetas <- theta_av(irt$theta)

```

We can now begin our analysis, and we start by bringing in additional variables from the original dataset into the analysis. The IRT-M code itself does not keep unique ids for responses; however, it retains the order of the rows of the input data. This allows one to attach other variables to the matrix of `theta` estimates, `avgthetas`. After bringing in the additional variables, we then specify all column names of this new data frame (`thetas'). This provides us a data frame that can be used for inferential and descriptive analysis.

In the context of the synthetic survey that forms this vignette, we can bring in both metadata and survey instruments themselves. More broadly, we could connect the estimated thetas to any respondent-level metadata to which we had access, though that might require assigning codes to each respondent and merging data.

Here we read in a data frame with demographic data from the survey, and bind that to the theta averages. `synth_idvs' is a table with a synthetic version of independent variables from the underlying survey data. It takes the large survey dataset and extracts respondent-level questions that we're interested in for the visualizations: respondent unique identifiers and country, their self-reported socioeconomic class, and their self-reported political orientation. In the following code, we import the data and attach it to our averaged respondent-level theta estimates. We also make the column names more human-readable. At this stage, we convert several of the metadata columns to factors, to facilitate the visualizations.

We finish by calling R's `head()` method to preview the data.

```{r read-idvs, eval=T, echo=T}

## load idvs:
load("./vdata/synth_idvs.rda")

thetas <- cbind(avgthetas, synthidvs)

## Rename columns for readability:
colnames(thetas)[1:6] <- paste0("Theta", 1:6)
colnames(thetas)[colnames(thetas)=="qb7_2"] <- "MoreBorderControl"

## Cast into factors:
thetas$mediatrust <- as.factor(thetas$mediatrust)
thetas$class <- as.factor(thetas$class)
thetas$polorient <- as.factor(thetas$polorient)

head(thetas)

```

## Correlation matrices

One difference between IRT-M and other conventional IRT approaches is that IRT-M assumes that the latent dimensions may be correlated, as is true for many theoretical concepts, and it learns the covariance matrix for the latent dimensions. It can be informative, as an initial descriptive analysis, to examine the correlation matrix between the latent dimensions. The function `dim_corr()` does that. It takes as input the array of covariance matrices output by IRT-M (either in `Sigma` or in `Omega`), averages their elements across samples, and computes and returns a correlation matrix of dimension `d x d`. The function also takes an optional vector input, `dim_names`, comprising names of each dimension. If that input is present and of length d, it adds corresponding row and column names to the correlation matrix before returning it.

```{r corr-matrixes, eval=T, echo=T, message=F, warning = FALSE}

#Compute correlation matrix of latent dimensions
theta_names <- c("Culture Threat", "Religion Threat", "Economic Threat", "Health Threat", "Support Immigration", "Support EU")
theta_corr <- dim_corr(irt$Sigma, theta_names)
theta_corr

```

## Visualizations

At this point, the analyst will likely want to visualize distributions over each latent dimension, possibly subset by other variables. The IRT-M package makes this easy to do with the function `irt_vis()`. This function takes as input the number of latent dimensions, `d`, and a data frame (`Y_out`) constructed above, containing the average thetas plus any additional variables supplied by the analyst. It returns a ridge plot illustrating the population distribution over each latent dimension.  The function also takes two optional arguments. The first, `sub_name`, takes the name of another variable in `Y_out` and provides subplots for each level in the variable `sub_name`. The second, `out_file`, takes a file name; if one is provided, the function writes the plot to a file with that name and places it in the working directory. Both default to `NULL` if not provided.

The following code presents examples of visualizations for the latent dimension estimations. We focus on the subgroup distribution visualizations presented in Morucci et al. 2024. Earlier, we loaded packages to assist in visualization. `irt_vis()` makes use of a  `melt()` call from the  `dplyr` library, which converts the data from wide to long format. Treating each `theta` identifier as a factor makes it easy to use the `ggridges` library to visualize ridge plots for each latent dimension. Before running `irt_vis()`, we rename the columns for the latent dimensions of `thetas` (its first `d` columns), since we are plotting the constrained model only. We first call `irt_vis()` without using a `sub_name` variable to produce aggregate distributions. We do pass a name for an output file.

```{r viz-thetas, eval=T, echo=T, echo=T, message=F, warning = FALSE}
library(ggplot2) #version: ggplot2_3.4.4
library(ggridges) #version: ggridges_0.5.6
library(RColorBrewer) #version: RColorBrewer_1.1-3
library(dplyr) #version: dplyr_1.1.4
library(ggrepel) # version: ggrepel_0.9.5
library(reshape2) #version: reshape2_1.4.4


## Rename for interpretability:
## Mapping:
## Theta1-Culture threat
## Theta2-ReligionThreat
## Theta3-Economic Threat
## Theta4-HealthThreat
## Theta5-OutcomeSupportImmigration
## Theta6-OutcomeSupportEU

colnames(thetas)[1:6] <-  recode(colnames(thetas)[1:6],
                                 "Theta1" = "Culture Threat",
                                 "Theta2" = "Religion Threat",
                                 "Theta3" = "Economic Threat",
                                 "Theta4" = "Health Threat",
                                 "Theta5" = "Support Immigration",
                                 "Theta6" = "Support EU")

#Save aggregate plot
ggbase <- irt_vis(d = d, T_out = thetas, sub_name = NULL, out_file = "ebirtm-synth.png")

```

![Visualization of estimated Theta distributions, Eurobarometer Data](ebirtm-synth.png){width=50%}

Now we turn to more sophisticated visualizations. For example, we may have a theory in which we expect that the underlying dimensions we find will be moderated by demographic variables.  We can use the metadata that we retained for the visualizations. The following example code presents ridge plot visualizations for different levels of respondent-reported media trust patterns (column 17 in the data):

```{r medtrust-viz, eval=T, echo=T, message=F, warning = FALSE}

#Save plot subset by media trust
ggmt <- irt_vis(d = d, T_out = thetas, sub_name = "mediatrust", out_file = "theta-media-synth.png")

```

![Visualization of the distribution of Thetas by media trust.](theta-media-synth.png){width=50%}

## Lambda

After estimating and exploring the theta distributions, one might turn to questions of validation. The IRT-M model produces output that can help with this task: the `lambda` array records how closely associated each item is to each latent dimension, including any outcome latent dimensions.

IRT-M comes with a utility, `get_lambdas()`, to make using the information in that array a bit easier. It takes three inputs: the `lambda` array from the IRT model, a vector of item names, and a vector of latent dimension names. The utility returns a list with two objects, both of which are data frames. The first, `av_lams`,  contains averages (across samples) of item discrimination parameters. The second, `high_lams`, is a `K x d` matrix, with each column an ordered list of the items (by name) that most strongly load on each latent dimension. The utility also accepts an optional argument, `item_elab`, which is a vector of item descriptions. If that argument is not null (its default), then the utility appends those descriptions to the item names in its output.

For our vignette, `lambda` is an array where the first dimension is the number of one-hot encoded survey questions, the second dimension is the number of latent dimensions, and the third dimension is the number of thinned samples produced by the IRT-M sampler. Our MCodes matrix also contains substantive notes for each item. The code below returns useful information about model loadings.

```{r lambdas, eval=TRUE, echo=T, message=F, warning = FALSE}

#Extract relevant substantive notes and create data frame with them and item codes
filtered_MCodes <- MCodes[MCodes[[2]] %in% M_matrix$QMap, , drop = FALSE]
M_df <- data.frame(QMap = M_matrix$QMap, sn = filtered_MCodes[[3]])

#Explore item loadings
lambdas <- get_lambdas(irt$lambda, item_names = M_df$QMap, dim_names = theta_names, item_elab = M_df$sn)
average_lambdas <- lambdas[[1]]
highest_lambdas <- lambdas[[2]]
average_lambdas
highest_lambdas

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/irt_m.R
\name{irt_m}
\alias{irt_m}
\title{irt_m}
\usage{
irt_m(
  Y_in,
  d,
  M_matrix = NULL,
  nburn = 1000,
  nsamp = 1000,
  thin = 1,
  learn_loadings = FALSE
)
}
\arguments{
\item{Y_in}{a N x K matrix of responses given by N respondents to K items. Can contain missing values. Column names should match first column in M_matrix.}

\item{d}{an integer specifying the number of latent dimensions.}

\item{M_matrix}{a K x (d+1) matrix of theoretical codings used to constrain IRT-M  (default=NULL). First column should match column names in Y_in.}

\item{nburn}{an integer specifying the number of burn-in MCMC iterations.}

\item{nsamp}{an integer specifying the number of sampling MCMC iterations.}

\item{thin}{an integer specifying the number of thinning MCMC samples.}

\item{learn_loadings}{a Boolean specifying whether a covariance matrix for the latent loadings should be learned, instead of the default covariance matrix for latent dimensions.}
}
\value{
A list containing the following components:
\item{lambda}{An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.}
\item{b}{A matrix of dimension (K x nsamp/thin) containing posterior samples of item difficulty parameters.}
\item{theta}{An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values.}
\item{Sigma}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of latent traits (only if learn_Sigma=TRUE).}
\item{Omega}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of item loadings (only if learn_Omega=TRUE).}
}
\description{
This function is a wrapper to enable easier use of the IRT-M model in M_constrained_irt.
It takes as input two data frames: a N x K data frame, and a K x (1+d) M-matrix.
The first column of the M-matrix should contain item identifiers that match the K column headers in
the N x K data frame. If they do not match, the wrapper exits with an error.
The wrapper computes anchors, Y_all (merged data and anchors), and a list of diagonal M-Matrices.
The second two are used as inputs to M_constrained_irt, which runs the sampler. Also used as input are
nburn (Default = 10^3), nsamp (Default = 10^3), thin (Default = 1), and
learn_loadings (Default = FALSE). This last one defaults to having the sampler learn factor covariances.
If set to true, it will learn loading covariances instead.
Finally, the wrapper removes the anchors and returns an irt list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{synth_questions}
\alias{synth_questions}
\title{Questions for the Synthetic European sentiment survey in the vignette}
\format{
A data frame with 3000 rows and 148 columns representing synthetic survey responses
}
\description{
A synthetic dataset with 3000 rows and 148 questions.
This data replicates the structure of questions following Eurobarometer 94.3 (2021).
It is not intended to be analyzed independently from the vignette.
'
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_analysis.R
\name{irt_vis}
\alias{irt_vis}
\title{irt_vis}
\usage{
irt_vis(d, T_out, sub_name = NULL, out_file = NULL)
}
\arguments{
\item{d}{The number of latent dimensions}

\item{T_out}{N x (d+z) data frame with average latent dimensions in first d columns}

\item{sub_name}{The name of a variable in T_out used for levels in the plot (Default = NULL)}

\item{out_file}{Output file name for plot (Default = NULL)}
}
\value{
A ggplot2 object containing density ridge plots of the latent dimensions. When sub_name is NULL, the plot shows the distribution of each theta dimension. When sub_name is provided, the plot shows distributions faceted by theta dimension and grouped by the specified variable.
}
\description{
Takes as input the number of latent dimensions (d),
an N x (d+z) data frame with average thetas in the first d columns and variables not included in the
calculation of the thetas in the rest (T_out), and, optionally, a variable name (sub_name)
taken from T_out, and an output file name (out_file),
and returns either unconditional theta distributions or distributions subset by that variable
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{synth_idvs}
\alias{synth_idvs}
\title{Synthetic Independent Variables}
\format{
A 3000 row and 27 column dataset of synthetic survey responses. This closely follows the 94.3 Eurobarometer survey in structure.
This dataset is a toy that is intended for the IRT-M vignette. For real analysis, see the original Eurobarometer data collection.
}
\description{
A synthetic dataset of independent variables for post-estimate analysis in the vignette.
Extraction of the data from the synthetic survey is described in the vignette.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_analysis.R
\name{theta_av}
\alias{theta_av}
\title{Average Thetas}
\usage{
theta_av(theta_array)
}
\arguments{
\item{theta_array}{An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values}
}
\value{
theta_av N x d matrix of average thetas
}
\description{
Compute matrix of theta means over posterior distributions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{theta_lambda_traceplots}
\alias{theta_lambda_traceplots}
\title{Theta Lambda Traceplots}
\usage{
theta_lambda_traceplots(irt, i = NULL, k = NULL)
}
\arguments{
\item{irt}{An object containing theta and lambda parameters from an IRTM model}

\item{i}{Index of the respondent to plot (randomly selected if NULL)}

\item{k}{Index of the item to plot (randomly selected if NULL)}
}
\value{
Plots of theta, lambda, and their product across MCMC iterations
}
\description{
Creates traceplots for IRT parameter convergence diagnostics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constrained_IRT.R
\name{M_constrained_irt}
\alias{M_constrained_irt}
\title{M_constrained_irt}
\usage{
M_constrained_irt(
  Y,
  d,
  M = NULL,
  theta_fix = NULL,
  which_fix = NULL,
  nburn = 1000,
  nsamp = 1000,
  thin = 10,
  learn_Sigma = TRUE,
  learn_Omega = FALSE,
  hyperparameters = list(),
  display_progress = TRUE
)
}
\arguments{
\item{Y}{a N x K matrix of responses given by N respondents to K items. Can contain missing values.}

\item{d}{an integer specifying the number of latent dimensions.}

\item{M}{a list of K d x d matrices (default=NULL).}

\item{theta_fix}{a matrix with d columns containing the values of the latent dimensions for respondents that have pre-specified latent factors.}

\item{which_fix}{a vector containing the indices of the respondents for which latent factors have been fixed.}

\item{nburn}{an integer specifying the number of burn-in MCMC iterations.}

\item{nsamp}{an integer specifying the number of sampling MCMC iterations.}

\item{thin}{an integer specifying the number of thinning MCMC samples.}

\item{learn_Sigma}{a Boolean specifying whether a covariance matrix for the latent factors should be learned.}

\item{learn_Omega}{a Boolean specifying whether a covariance matrix for the latent loadings should be learned.}

\item{hyperparameters}{a list of hyperparameters for the model.}

\item{display_progress}{a Boolean specifying whether a progress bar should be displayed.}
}
\value{
A list containing the following components:
\item{lambda}{An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.}
\item{b}{A matrix of dimension (K x nsamp/thin) containing posterior samples of item difficulty parameters.}
\item{theta}{An array of dimension (N x d x nsamp/thin) containing posterior samples of respondent latent trait values.}
\item{Sigma}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of latent traits (only if learn_Sigma=TRUE).}
\item{Omega}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the covariance matrix of item loadings (only if learn_Omega=TRUE).}
}
\description{
This function allows you to run the IRT model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_analysis.R
\name{dim_corr}
\alias{dim_corr}
\title{learned correlations}
\usage{
dim_corr(cov_array, dim_names = NULL)
}
\arguments{
\item{cov_array}{An array of dimension (d x d x nsamp/thin) containing posterior samples of the relevant covariance matrix.}

\item{dim_names}{Vector of dimension names.}
}
\value{
A data frame containing the correlation matrix derived from t input covariance array, with rows and columns labeled according to dim_names (if provided). Each cell represents the correlation between the corresponding dimensions.
}
\description{
Takes as input either the Sigma covariance matrix, if the user has learned the factor covariance,
or the Omega covariance matrix, if the user has learned the loading covariance, as well as a vector of
dimension names.
Returns a correlation matrix with correlations between the dimensions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{standardize_theta}
\alias{standardize_theta}
\title{Standardize Theta}
\usage{
standardize_theta(theta, Sigma)
}
\arguments{
\item{theta}{estimated object}

\item{Sigma}{covariance matrix}
}
\value{
theta divided by sigma param
}
\description{
standardizes theta estimates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{mse}
\alias{mse}
\title{Mean Squared Error}
\usage{
mse(ytrue, ypred, aggregate = TRUE, root = FALSE)
}
\arguments{
\item{ytrue}{observed values,}

\item{ypred}{predicted values}

\item{aggregate}{logical for whether to take mean of estimate}

\item{root}{logical for whether to return square root of MSE}
}
\value{
mean squared error
}
\description{
loss function for benchmarks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_analysis.R
\name{get_lambdas}
\alias{get_lambdas}
\title{get_lambdas}
\usage{
get_lambdas(lambda_array, item_names, dim_names, item_elab = NULL)
}
\arguments{
\item{lambda_array}{An array of dimension (K x d x nsamp/thin) containing posterior samples of item discrimination parameters.}

\item{item_names}{Vector of item names.}

\item{dim_names}{Vector of dimension names.}

\item{item_elab}{A vector comprising elaborations about each item (Default = NULL).}
}
\value{
A list containing the following components:
\item{av_lams}{A data frame of dimension (K x (1+d)) containing averages of item discrimination parameters.}
\item{high_lams}{A data frame of dimension (K x d) containing an ordered list of the items with the highest mean values of lambda for each dimension.}
}
\description{
Takes as input the array of lambdas from the irt list,
a vector of item names (can be taken from either Y_in or M_matrix), a vector of dimension names,
and, optionally, a vector comprising elaborations about each item.
Returns a list containing a data frame with the mean lambdas for each item-dimension pair,
possibly attaching elaborations to each item's string,
and a data frame with the items with the highest mean values of lambda for each dimension in order
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{Geweke_convergence}
\alias{Geweke_convergence}
\title{Geweke Convergence}
\usage{
Geweke_convergence(THETA)
}
\arguments{
\item{THETA}{Matrix of parameter estimates from IRTM}
}
\value{
Proportion of values that fail the Geweke convergence test (p < 0.05) for each parameter
}
\description{
Runs Geweke tests to assess MCMC convergence
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MCodes}
\alias{MCodes}
\title{Methodological Codes}
\source{
IRT-M vignette walk through.
}
\description{
Factor loading matrix for IRT-M vignette. This is a 793 row and 9 column dataset.
The rows are derived from the binary encoding of the synthetic survey, with a row for every
binarized question in the survey. The first 56 rows are retained metadata, and have lots of NA.

The data format is an intermediary processing for IRT-M, and is detailed in the vignette text.

@format A data frame with the following variables:
#' \describe{
\item{QCode}{Mapping of the dimension coding key to the underlying question in the original (synthetic) survey data.
The first 56 rows are blank because they map to survey and respondent metadata that doesn't relate to dimensions.}
\item{QMap}{Mapping to the question in data processed by the vignette for variable estimation. }
\item{SubstantiveNotes}{Brief human readable comments on the substantive meaning of the coded questions. These are for convenience of reference.}
\item{D1-Culture threat}{Loading vector for the cultural threat dimension. A 1 indicates that the question is expected to load.}
\item{D2-ReligionThreat}{Loading vector for the religious threat dimension.}
\item{D3-Economic Threat}{Loading vector for the economic threat dimension.}
\item{D4-HealthThreat}{Loading vector for the health threat dimension.}
\item{O1-OutcomeSupportImmigration}{Loading vector for the immigration support composite.}
\item{O2-OutcomeSupportEU}{Loading vector for the European Union support composite.}
}
}
\details{
Datasets for IRT-M Package

MCodes, synth_idvs, and synth_questions are included in the vignette
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pair_gen_anchors.R
\name{pair_gen_anchors}
\alias{pair_gen_anchors}
\title{pair_gen_anchors}
\usage{
pair_gen_anchors(M, A)
}
\arguments{
\item{M}{a list containing K dxd M-matrices}

\item{A}{What value should be considered extreme for the latent dimensions.}
}
\value{
A list with two elements:
\item{Yfake}{A matrix of dimension (d(d-1)*4 x K) containing the imputed answers of the fake anchor respondents, where d is the number of dimensions and K is the number of questions. Values are 0 (no) or 1 (yes).}
\item{theta_fake}{A list of d(d-1)*4 vectors, each of length d, representing the latent trait values for the fake anchor respondents. Each vector contains mostly zeros, with extreme values (A or -A) at the positions corresponding to the pair of dimensions being considered.}
}
\description{
This function generates anchor points from the M matrices.
It creates d(d-1)*4 fake respondents such that, for every pair of dimensions:
The first respondent has an extremely positive value of both dimensions of the pair,
The second has an extremely positive value for dim 1 and an extremely negative value for dim 2 of each pair,
The third has an extremely negative value for dim 1 and an extremely positive value for dim 2 of each pair,
The fourth has an extremely negative value of both dimensions of the pair.
These respondents' answers are imputed according to the directions of loadings specified by the M-matrices, i.e.,
if question k loads positively on dim 1 and positively on dim 2, the first respondent in the dim 1/dim 2 pair will have
yes for question k. If question k+1 loads  positively on dim 1 and negatively on dim 2 then the second respondent in
the dim 1/dim 2 pair will have yes on question k+1 and so on.
}
